@@ -5,7 +5,8 @@ use pokey_logger::existing_log_handler::ExistingLogHandler;
 use pokey_logger::{Logger, LOGGER};
 
 fn main() {
-    // Load a configuration file
+    // Load a configuration file if the config feature is enabled.
+    #[cfg(feature = "config")]
     match LOGGER.load_config_file("examples/full_usage/config.yml") {
         Ok(_) => info!("Config file loaded"),
         Err(e) => error!("Error loading config file: {e:?}")
@@ -33,6 +34,8 @@ fn main() {
 /// then the actual log file will be overwritten.
 fn file_renaming() {
     let logger = Logger::new();
+    #[cfg(feature = "env")]
+    logger.load_env_vars();
     logger.set_log_path("examples/full_usage/logs/rename_log.log");
     logger.set_existing_log_handler(ExistingLogHandler::Rename);
     logger.info("This is cool");
