@@ -3,7 +3,7 @@
 //! ## Usage
 //! First, the library must be added to the project's `Cargo.toml` file.
 //! ```toml
-//! pokey_logger = "0.2.0"
+//! pokey_logger = "0.3.0"
 //! ```
 //! or to get the latest and greatest
 //! ```toml
@@ -28,6 +28,7 @@
 //!     // are shown in the terminal
 //!     LOGGER.set_color(true);
 //!     LOGGER.set_level(Level::Debug);
+//!     #[cfg(feature = "log_files")]
 //!     if !LOGGER.set_log_path("logs/server.log") {
 //!         warn!("Could not set log path");
 //!     }
@@ -50,6 +51,71 @@
 //! its settings from multiple threads. If you would like separate configurations
 //! and instances, the `Logger` struct itself can be instantiated and passed around
 //! as the developer sees fit.
+//!
+//! # Configuration
+//!
+//! There are three main ways to configure the logger.
+//! 1. Directly through functions on the logger.
+//! 2. A config file using the `config` feature.
+//! 3. Using environment variables.
+//!
+//! ## Directly
+//!
+//! Look at the documentation either through docs.rs or through the `cargo doc`
+//! command to find all the functions that can be called to configure the
+//! logger at run time.
+//!
+//! ## Config File
+//!
+//! After starting your program, you can tell the logger to load a yaml config
+//! file to configure the logger. Here's an example config file:
+//! ```yaml
+//! level: Debug
+//! color: true
+//! time_stamp: true
+//! file_color: false
+//! log_file_path: "./log/development.log"
+//! existing_log_handler: Append
+//! ```
+//!
+//! To load it, see the [`Logger::load_config_file()`] method.
+//!
+//! **NOTE:** You will need to enable the `config` feature to do this. This
+//! might look something like this in your `Cargo.toml` file.
+//! ```toml
+//! pokey_logger = { version = "0.3.0", features = ["config"] }
+//! ```
+//! ## Environment Variables
+//!
+//! The third way to configure the logger is through environment variables. For
+//! full documentation on all the variables and what they do, see the
+//! [`environment::configure`] method. The `env` feature must be enabled to use
+//! environment variables, however this feature is on by default.
+//!
+//! If you are using macros and the global [`static@LOGGER`] variable, then there is
+//! no need to do anything special, as the environment variables are
+//! automatically applied. To apply the environment variables to a custom
+//! logger, calling the [`Logger::load_env_vars`] function will load them.
+//!
+//! In a command this might look like:
+//! ```text
+//! PL_LEVEL=info PL_COLOR=false some_program
+//! ```
+//!
+//! # Features
+//!
+//! There are a few features that can be turned off when using the crate to
+//! build a smalled binary.
+//!
+//! ## Default Features
+//!
+//! - **time** - Use `chrono` to put the time at the beginning of messages.
+//! - **log_files** - Allow log files to be saved.
+//! - **env** - Allow the use of environment variables to configure the logger.
+//!
+//! ## Optional Features
+//!
+//! - **config** - Allows loading of a .yml config file. Includes `serde`.
 
 #![allow(dead_code)]
 // Allow needless doctest main function because example above makes more sense
@@ -61,30 +127,51 @@ mod tests;
 #[macro_use]
 pub mod logging_macros;
 pub mod color;
+#[cfg(feature = "log_files")]
 pub mod existing_log_handler;
+#[cfg(feature = "time")]
 pub mod time;
 
+#[cfg(feature = "config")]
 mod config_file;
+#[cfg(feature = "env")]
+pub mod environment;
 mod level; // not public because level is reexported
 mod log_message;
 
+#[cfg(feature = "config")]
 pub use config_file::ConfigFileLoadError;
+
 pub use level::Level;
 
+#[cfg(feature = "log_files")]
 use crate::existing_log_handler::ExistingLogHandler;
-use color::TermColor;
+#[cfg(feature = "log_files")]
+use std::fs::File;
+#[cfg(feature = "log_files")]
+use std::io::{prelude::*, BufWriter};
+#[cfg(feature = "log_files")]
+use std::path::PathBuf;
+
+#[cfg(feature = "config")]
 use config_file::ConfigFile;
+
+use color::TermColor;
 use lazy_static::lazy_static;
 use log_message::LogMessage;
-use std::fs::File;
-use std::io::{prelude::*, BufWriter};
-use std::path::PathBuf;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Mutex;
 
 lazy_static!(
     /// The global logger.
-    pub static ref LOGGER: Logger = Logger::new();
+    pub static ref LOGGER: Logger = {
+        let mut logger = Logger::new();
+
+        #[cfg(feature = "env")]
+        environment::configure(&mut logger);
+
+        logger
+    };
 );
 
 // TODO: Scoped references that are basically references for certain files and
@@ -108,20 +195,26 @@ lazy_static!(
 /// not require mutable access to the logger because all of the settings are
 /// behind a mutex or atomic boolean.
 /// ```rust
-/// use pokey_logger::{Level, LOGGER, warn, ConfigFileLoadError};
+/// use pokey_logger::{Level, LOGGER, warn};
+/// #[cfg(feature = "config")]
+/// use pokey_logger::ConfigFileLoadError;
 /// // Only log debug and above. default is info
 /// LOGGER.set_level(Level::Debug);
 /// // Turn off colors. default is true
 /// LOGGER.set_color(false);
 /// // Turn off the timestamp. default is true
+/// #[cfg(feature = "time")]
 /// LOGGER.set_should_show_time(false);
 /// // Set the log path. default is none. See remove_log_path
+/// #[cfg(feature = "log_files")]
 /// if !LOGGER.set_log_path("logs/server.log") {
 ///     warn!("Could not set log path");
 /// }
 /// // Whether or not to put colour code in the log file. default is false
+/// #[cfg(feature = "log_files")]
 /// LOGGER.set_log_file_color(true);
 /// // Or even load a configuration file
+/// #[cfg(feature = "config")]
 /// if let Err(e) = LOGGER.load_config_file("config/logger.yml") {
 ///    warn!("Could not load config file: {e:?}");
 /// }
@@ -129,11 +222,17 @@ lazy_static!(
 pub struct Logger {
     level: Mutex<Level>,
     color: AtomicBool,
+    #[cfg(feature = "log_files")]
     log_file_color: AtomicBool,
+    #[cfg(feature = "time")]
     show_time: AtomicBool,
+    #[cfg(feature = "log_files")]
     log_path: Mutex<Option<PathBuf>>,
+    #[cfg(feature = "log_files")]
     log_writer: Mutex<Option<BufWriter<File>>>,
+    #[cfg(feature = "log_files")]
     existing_log_handler: Mutex<ExistingLogHandler>,
+    #[cfg(feature = "time")]
     timestamp_format: Mutex<Option<String>>
 }
 
@@ -145,15 +244,29 @@ impl Logger {
         Logger {
             level: Mutex::new(Level::Debug),
             color: AtomicBool::new(true),
+            #[cfg(feature = "log_files")]
             log_file_color: AtomicBool::new(false),
+            #[cfg(feature = "time")]
             show_time: AtomicBool::new(true),
+            #[cfg(feature = "log_files")]
             log_path: Mutex::new(None),
+            #[cfg(feature = "log_files")]
             log_writer: Mutex::new(None),
+            #[cfg(feature = "log_files")]
             existing_log_handler: Mutex::new(ExistingLogHandler::Overwrite),
+            #[cfg(feature = "time")]
             timestamp_format: Mutex::new(None)
         }
     }
 
+    /// Load all the environment variables available for the logger.
+    ///
+    /// This is done automatically on the global instance.
+    #[cfg(feature = "env")]
+    pub fn load_env_vars(&self) {
+        environment::configure(self);
+    }
+
     /// Set the log level. Only logs with a level equal to or higher than the
     /// set level will be logged.
     pub fn set_level(&self, level: Level) {
@@ -183,34 +296,40 @@ impl Logger {
 
     /// Set whether or not the logger should use colors in the log file. True
     /// means use colors, false means don't use colors.
+    #[cfg(feature = "log_files")]
     pub fn set_log_file_color(&self, color: bool) {
         self.log_file_color.store(color, Ordering::Relaxed);
     }
 
     /// Get whether or not the logger should use colors in the log file. True
     /// means use colors, false means don't use colors.
+    #[cfg(feature = "log_files")]
     pub fn get_log_file_color(&self) -> bool {
         self.log_file_color.load(Ordering::Relaxed)
     }
 
     /// Set how existing log files should be handled.
+    #[cfg(feature = "log_files")]
     pub fn set_existing_log_handler(&self, handler: ExistingLogHandler) {
         *self.existing_log_handler.lock().unwrap() = handler;
     }
 
     /// Get how existing log files should be handled.
+    #[cfg(feature = "log_files")]
     pub fn get_existing_log_handler(&self) -> ExistingLogHandler {
         *self.existing_log_handler.lock().unwrap()
     }
 
     /// Set whether or not the logger should show the timestamp. True means
     /// show the timestamp, false means don't show the timestamp.
+    #[cfg(feature = "time")]
     pub fn set_should_show_time(&self, show_time: bool) {
         self.show_time.store(show_time, Ordering::Relaxed);
     }
 
     /// Get whether or not the logger should show the timestamp. True means
     /// show the timestamp, false means don't show the timestamp.
+    #[cfg(feature = "time")]
     pub fn should_show_time(&self) -> bool {
         self.show_time.load(Ordering::Relaxed)
     }
@@ -235,6 +354,7 @@ impl Logger {
     /// # Returns
     /// The return value will be `true` if the path is successfully set, or
     /// `false` if could not set the path.
+    #[cfg(feature = "log_files")]
     pub fn set_log_path(&self, path: &str) -> bool {
         let path_buf = PathBuf::from(path);
         self.remove_log_writer();
@@ -258,17 +378,20 @@ impl Logger {
     }
 
     /// Remove file logging.
+    #[cfg(feature = "log_files")]
     pub fn remove_log_path(&self) {
         *self.log_path.lock().unwrap() = None;
         self.remove_log_writer();
     }
 
     /// Get the path to the file that the logger is logging to.
+    #[cfg(feature = "log_files")]
     pub fn get_log_path(&self) -> Option<PathBuf> {
         (*self.log_path.lock().unwrap()).as_ref().cloned()
     }
 
     /// Get the format of the timestamp on log messages.
+    #[cfg(feature = "time")]
     pub fn get_timestamp_format(&self) -> Option<String> {
         let res = match self.timestamp_format.lock() {
             Ok(ref mut inner) => inner.clone(),
@@ -279,6 +402,7 @@ impl Logger {
     }
 
     /// Set the format of the timestamp on log messages.
+    #[cfg(feature = "time")]
     pub fn set_timestamp_format(&self, value: Option<String>) {
         match self.timestamp_format.lock() {
             Ok(mut inner) => *inner = value,
@@ -288,11 +412,13 @@ impl Logger {
 
     /// Set the file writer to write actual data to. This method should only
     /// be called internally.
+    #[cfg(feature = "log_files")]
     fn set_log_writer(&self, buf_writer: BufWriter<File>) {
         *self.log_writer.lock().unwrap() = Some(buf_writer);
     }
 
     /// Remove the file writer. This method should only be called internally.
+    #[cfg(feature = "log_files")]
     fn remove_log_writer(&self) {
         *self.log_writer.lock().unwrap() = None;
     }
@@ -300,6 +426,7 @@ impl Logger {
     /// Whether or not the logger writer has already been set. If this method
     /// is false but log path is set, a log writer should be created.
     /// See [`set_log_writer_if_not_set`](#method.set_log_writer_if_not_set)
+    #[cfg(feature = "log_files")]
     fn has_log_writer(&self) -> bool {
         if let Ok(lw) = self.log_writer.lock() {
             return lw.is_some();
@@ -309,6 +436,7 @@ impl Logger {
     }
 
     /// If there is a log path set, but no log writer, create a log writer.
+    #[cfg(feature = "log_files")]
     fn set_log_writer_if_not_set(&self) {
         if !self.has_log_writer() {
             if let Some(path) = self.get_log_path() {
@@ -326,14 +454,8 @@ impl Logger {
         }
     }
 
-    /// Actually write the log message to the file and stdout. Should only be
-    /// called internally by the `debug`, `info`, `warn`, and `error` methods.
-    fn log_message(&self, level: Level, message: &str) {
-        let mut log_message = LogMessage::new(&self.prefix(), message, level);
-
-        // Print to stdout
-        print!("{}", log_message.formatted(self.get_color()));
-
+    #[cfg(feature = "log_files")]
+    fn log_message_to_file(&self, log_message: &mut LogMessage) {
         // Write to file
         self.set_log_writer_if_not_set();
         if let Ok(ref mut log_writer) = self.log_writer.lock() {
@@ -353,8 +475,23 @@ impl Logger {
         }
     }
 
-    // TODO: For now it just prints to stdout. In the future it should be
-    //       able to print to a file and should do stuff asynchronously.
+    #[cfg(not(feature = "log_files"))]
+    fn log_message_to_file(&self, _msg: &mut LogMessage) {
+        // Intentionally do nothing when the feature is not enabled
+    }
+
+    /// Actually write the log message to the file and stdout. Should only be
+    /// called internally by the `debug`, `info`, `warn`, and `error` methods.
+    fn log_message(&self, level: Level, message: &str) {
+        let mut log_message = LogMessage::new(&self.prefix(), message, level);
+
+        // Print to stdout
+        print!("{}", log_message.formatted(self.get_color()));
+
+        #[cfg(feature = "log_files")]
+        self.log_message_to_file(&mut log_message);
+    }
+
     /// Print a message to the log at the `debug` level. Will only print if the
     /// `debug` level is enabled.
     pub fn debug(&self, message: &str) {
@@ -390,6 +527,7 @@ impl Logger {
     /// The prefix to be added to all log messages. Currently this is just the
     /// timestamp, but in the future it could be something like the scope of
     /// the log message.
+    #[cfg(feature = "time")]
     fn prefix(&self) -> String {
         if self.should_show_time() {
             time::current_time_box(self.get_timestamp_format())
@@ -398,8 +536,17 @@ impl Logger {
         }
     }
 
+    /// The prefix to be added to all log messages. Currently this is just the
+    /// timestamp, but in the future it could be something like the scope of
+    /// the log message.
+    #[cfg(not(feature = "time"))]
+    fn prefix(&self) -> String {
+        "".to_string()
+    }
+
     /// Ensure all io buffers are cleared; usually before shutdown.
     pub fn flush(&self) -> std::io::Result<()> {
+        #[cfg(feature = "log_files")]
         if let Ok(ref mut log_writer) = self.log_writer.lock() {
             if log_writer.is_some() {
                 log_writer.as_mut().unwrap().flush()
@@ -409,6 +556,9 @@ impl Logger {
         } else {
             Ok(())
         }
+
+        #[cfg(not(feature = "log_files"))]
+        Ok(())
     }
 
     /// Loads all settings from a config file.
@@ -423,13 +573,18 @@ impl Logger {
     ///
     /// LOGGER.load_config_file("/examples/full_usage/config.yml");
     /// ```
+    #[cfg(feature = "config")]
     pub fn load_config_file(&self, path: &str) -> Result<(), ConfigFileLoadError> {
         let config_file = ConfigFile::load(path)?;
         self.set_level(config_file.level);
         self.set_color(config_file.color);
+        #[cfg(feature = "time")]
         self.set_should_show_time(config_file.time_stamp);
+        #[cfg(feature = "log_files")]
         self.set_log_file_color(config_file.file_color);
+        #[cfg(feature = "log_files")]
         self.set_existing_log_handler(config_file.existing_log_handler);
+        #[cfg(feature = "log_files")]
         if let Some(ref log_path) = config_file.log_file_path {
             self.set_log_path(log_path);
         } else {
@@ -447,15 +602,3 @@ impl Default for Logger {
         Self::new()
     }
 }
-
-/// Set the log level of the global logger.
-#[deprecated(since = "0.2.0", note = "Use `set_level` on `LOGGER` instead")]
-pub fn set_level(level: Level) {
-    LOGGER.set_level(level);
-}
-
-/// Set whether or not the global logger should show colors.
-#[deprecated(since = "0.2.0", note = "Use `set_color` on `LOGGER` instead")]
-pub fn set_color(color: bool) {
-    LOGGER.set_color(color);
-}
